import { IDictionary } from '../utilities/common';
import { IObservable } from '../utilities/observable';
export interface TreeNode {
    isExpanded?: boolean;
    children?: Array<TreeNode>;
    hasChildren?: boolean;
    iconName?: string;
    iconTooltipContent?: string;
    iconClassName?: string;
    className?: string;
    isNodeDisabled?: boolean;
}
export declare type AugmentedTreeNode<T = {}> = TreeNode & T & {
    $meta: {
        nodeId?: any;
        parentNodeId?: any;
        nodeLevel: number;
        sortRequestId?: number;
        isLazyChildrenLoadInProgress?: boolean;
        isAsyncLoadingDummyNode?: boolean;
        satisfiesFilterCondition?: boolean;
        descendantSatisfiesFilterCondition?: boolean;
    };
    children?: Array<AugmentedTreeNode<T>>;
    parentNode: AugmentedTreeNode<T>;
};
export interface ITreeStructureRoot<T> {
    children?: Array<AugmentedTreeNode<T>>;
}
export interface ILookupTable {
    [id: number]: AugmentedTreeNode;
    [id: string]: AugmentedTreeNode;
}
export declare enum RootNodeBehaviourEnum {
    Ignore = 0,
    HideInUIKeepInDataStructure = 1,
    /**
     * NOT IMPLEMENTED
     */
    ShowInUI = 2,
}
export declare type IdGetter = (node: TreeNode) => string | number;
export declare type SelectedIdListener = (selectedIds: Array<string>) => void;
export declare type DataListener = (selectedIds: Array<AugmentedTreeNode>) => void;
/**
 * This class is meant to work with th TreeGrid component.
 * This class breaks the immutability principle of redux
 * but all mutation is done within this class and in conjunction with the TreeGrid component
 * This is done because of performance reasons
 * it is important to be aware of the fact that the input data for this datasource will be mutated
 * the mutation will occur when this class is created, also all operations that mutate the data.
 * All operations that mutate the data will return a new instance of TreeDataSource
 * this allows us to view the TreeDataSource as immutable when performing actions in our reducers
 * ie. we add a new child to the tree, react will register the change and the TreeGrid component will update because of the prop change
 */
export declare class TreeDataSource<T = {}> implements IObservable<React.Component> {
    private idCounter;
    private changeIteration;
    private treeStructure;
    private idMember;
    private renumberIds;
    private rootNodeBehaviour;
    isEmpty: boolean;
    /** Lookup that holds parent object for given node */
    /** Lookup that returns node object for given node id */
    private nodesById;
    /**
     *
     * @param input warning: will be mutated and returned as ITreeDataSource
     * @param idMember the field that contains the id of the node, or a function that returns a unique id for a node.
     * If no parameter is supplied ids will be generated automatically
     */
    constructor(input: TreeNode | TreeDataSource | Array<any>, idMember?: (string | IdGetter), enableRecursiveSelection?: boolean, selectedNodes?: Array<number | string>, rootNodeBehaviour?: RootNodeBehaviourEnum);
    private itemHasChildren;
    private extendNodes(parent, children);
    private extendSingleNode(node, parent);
    private getNodeId(node);
    private isDataSource(input);
    private isRootNodesArray(input);
    appendNode(node: T, parentNodeId?: number | string): TreeDataSource<T>;
    updateNode<NodeType = T>(nodeId: number | string, props: Partial<AugmentedTreeNode<NodeType>> | Partial<NodeType> & {
        children?: any;
    }): TreeDataSource<T>;
    private getNextSurogateId();
    getTreeStructure(): ITreeStructureRoot<T>;
    getNodeById<NodeType = T>(nodeId: number | string): AugmentedTreeNode<NodeType>;
    findNode<NodeType = T>(nodePredicate: (node: AugmentedTreeNode<NodeType>) => boolean): AugmentedTreeNode<NodeType>;
    private subscribers;
    private dataListeners;
    private selectedIdsListeners;
    enableRecursiveSelection: boolean;
    private selectedIds;
    private partiallySelectedIds;
    private _selectItemsStrategy;
    private _removeItemsStrategy;
    subscribe(listener: React.Component<{}, {}>): void;
    unsubscribe(listener: React.Component<{}, {}>): void;
    registerDataListener: (dataListener: DataListener) => void;
    removeDataListener: (dataListener: DataListener) => void;
    registerSelectedIdsListener: (selectedIdsListener: SelectedIdListener) => void;
    private notifyWithSelectedIds;
    private notifyWithSelectedNodes;
    setRecursiveSelection(isEnabled: boolean): void;
    notify: () => void;
    readonly SelectedNodes: IDictionary<boolean>;
    readonly PartiallySelectedNodes: IDictionary<boolean>;
    readonly NodesById: ILookupTable;
    setSelectedIds(items: Array<number | string>): void;
    selectAll(): void;
    deselectAll(): void;
    setSelected(item: AugmentedTreeNode): void;
    removeSelected(item: AugmentedTreeNode): void;
    private updateSelectStrategy(enableRecursiveSelection);
    private updateSelectedItems;
    private partiallySelectItem;
    private checkIfAllChildrenAreSelected;
    private removeSelectedItems;
    private removeSelectedItem;
    private setSelectedItem;
    private checkObject;
    private checkRecursive;
    /**
     * Return lookup table off all child ids
     */
    private recursiveChildSelection;
}
